# List of API endpoints PyDigitalEnergy Cloudbroker knows about

API_PATH = {
    # Compute
    'compute_list': '/cloudbroker/compute/list',
    'compute_get': '/cloudbroker/compute/get',
    # Stack
    'stack_list': '/cloudbroker/stack/list',
    'stack_get': '/cloudbroker/stack/get',
    # Node
    'node_list': '/cloudbroker/node/list',
    'node_get': '/cloudbroker/node/get',
    # Grid
    'grid_list': '/cloudbroker/grid/list',
    'grid_get': '/cloudbroker/grid/get',
    # Storage
    'sep_list': '/cloudbroker/sep/list',
    'sep_get': '/cloudbroker/sep/get',
    'sep_get_config': '/cloudbroker/sep/getConfig',
    'sep_get_pool': '/cloudbroker/sep/getPool',
    # Image
    'image_list': '/cloudbroker/image/list',
    'image_get': '/cloudbroker/image/get',
    # Disk
    'disks_list': '/cloudbroker/disks/list',
    'disks_get': '/cloudbroker/disks/get',
    # Network
    'extnet_list': '/cloudbroker/extnet/list',
    'extnet_get': '/cloudbroker/extnet/get',
}